;;;-*- Mode: Lisp; Package: (KANJI) -*-
;;;
;;;	basic-kanji-fred.lisp ver3.2
;;;
;;;	Masaya UEDA
;;;	ueda@shpscl.sharp.co.jp

(defpackage :kanji (:use :common-lisp :ccl))

(use-package :kanji :cl-user)

(in-package :kanji)

(eval-when (:compile-toplevel :load-toplevel :execute)
  (export '(basic-kanji-fred-mixin
            kanji-fred-window-mixin
            basic-kanji-listener
            basic-kanji-fred-window
            basic-kanji-fred-dialog-item
            kanji-fred-container
            *basic-kanji-fred-default-font-spec*)))

; ---

(declaim (inline kanji1p kanji2p buffer-before-kanjip
                 buffer-in-kanjip buffer-count-kanji-char
                 buffer-char-font-line-height mend))

(defun kanji1p (ch &aux (cd (char-code ch)))
  (declare (optimize (speed 3) (safety 0)))
  (or (<= #x81 cd #x9f) (<= #xe0 cd #xec)))

(defun kanji2p (ch &aux (cd (char-code ch)))
  (declare (optimize (speed 3) (safety 0)))
  (or (<= #x40 cd #x7e) (<= #x80 cd #xfc)))

(defun buffer-before-kanjip (buf pos)
  (declare (optimize (speed 3) (safety 0)))
  (kanji1p (buffer-char buf pos)))

(defun buffer-in-kanjip (buf &optional (pos (buffer-position buf))
                             &aux (foo pos))
  (declare (optimize (speed 3) (safety 0)))
  (loop (decf foo)
        (unless (and (<= 0 foo) (kanji1p (buffer-char buf foo)))
          (return)))
  (evenp (- pos foo)))

(defun buffer-count-kanji-char (fb cnt fwd)
  (declare (optimize (speed 3) (safety 0)))
  (let ((b (buffer-position fb)))
    (if fwd
      (let ((sz (buffer-size fb)))
        (do ((e b (+ e (if (buffer-before-kanjip fb e) 2 1))))
            ((or (<= sz e) (zerop cnt)) (- e b))
          (decf cnt)))
      (do ((e b (- e (if (buffer-in-kanjip fb (1- e)) 2 1))))
          ((or (<= e 0) (zerop cnt)) (- b e))
        (decf cnt)))))

(defun buffer-char-font-line-height (fb bp)
  (declare (optimize (speed 3) (safety 0)))
  (font-line-height (buffer-char-font-spec fb bp)))

(defun %mend (bkfm fb bp)
  (declare (optimize (speed 3) (safety 0)))
    (if (typep bkfm 'fred-dialog-item)
      (view-draw-contents (view-container bkfm))
      (let ((b (fred-vpos bkfm (- bp 2)))
            (l (fred-hpos bkfm (1- bp))))
        (rlet ((rect :rect
                     :left l
                     :top (- b (buffer-char-font-line-height fb (- bp 3)))
                     :bottom b
                     :right (- (* 2 l) (fred-hpos bkfm (- bp 3)))))
          (#_InValRect rect))
        (fred-update bkfm))))

(defun mend (bkfm fb bp)
  (declare (optimize (speed 3) (safety 0)))
  (if (and (< 2 bp)
           (zerop (logand #x80 (char-code (buffer-char fb (1- bp)))))
           (kanji2p (buffer-char fb (- bp 2)))
           (kanji1p (buffer-char fb (- bp 3))))
        (%mend bkfm fb bp)))

; ---

(defvar *basic-kanji-fred-default-font-spec* '("osaka|" 12))

(setq *mini-buffer-font-spec* '("Osaka|" 12 :PLAIN))

(setq *listener-default-font-spec* '("Osaka|" 12 :PLAIN))

(setq *default-editor-class* 'basic-kanji-fred-window)

(setq *default-listener-class* 'basic-kanji-listener)

(def-fred-command (:control #\l) invalidate-view)

; ---

(defclass basic-kanji-fred-mixin ()
  ((in-kanjip :initform nil :reader in-kanjip)
   (kanji-buffer :reader kanji-buffer :accessor kanji-buffer)
   (kanji-buffer-index :initform -1 :accessor kanji-buffer-index)
   (kanji-buffer-size :initarg :kanji-buffer-size :accessor kanji-buffer-size)
   (kanji-fred-prefix-argument :initform nil
                               :accessor kanji-fred-prefix-argument)))

(defclass kanji-fred-window-mixin () ()
  (:default-initargs :kanji-buffer-size #x80))

(defclass basic-kanji-listener (basic-kanji-fred-mixin kanji-fred-window-mixin
                                                       listener) ())

(defclass basic-kanji-fred-window (basic-kanji-fred-mixin kanji-fred-window-mixin
                                                          fred-window) ())

(defclass kanji-fred-container (window) ())

(defclass basic-kanji-fred-dialog-item (basic-kanji-fred-mixin fred-dialog-item) ()
  (:default-initargs :kanji-buffer-size #x10))

; ---

(defmethod initialize-instance :after ((bkfm basic-kanji-fred-mixin) &rest r
                                       &key kanji-buffer-size)
  (declare (dynamic-extent r) (ignore r))
  (setf (kanji-buffer bkfm) (make-array (+ kanji-buffer-size 2)
                                        :element-type 'character
                                        :fill-pointer 0)))

(defmethod view-default-font ((bkfm basic-kanji-fred-mixin))
  *basic-kanji-fred-default-font-spec*)

(defmethod ed-self-insert :before ((bkfm basic-kanji-fred-mixin))
  (if (kanji1p *current-character*)
    (setf (kanji-fred-prefix-argument bkfm) (fred-prefix-numeric-value bkfm)
          (fred-prefix-argument bkfm) nil)))

(defmethod flush-kanji-buffer ((bkfm basic-kanji-fred-mixin))
  (with-accessors ((buffer kanji-buffer) (index kanji-buffer-index)) bkfm
    (setf (fill-pointer buffer) (1+ index)
          index -1)
    (coerce buffer 'simple-string)))

(defmethod ed-insert-char :around ((bkfm basic-kanji-fred-mixin) ch)
  (declare (optimize (speed 3) (safety 0))
           (inline flush insert set-char))
  (labels ((flush ()
             (let* ((fb (fred-buffer bkfm)) (bp (buffer-position fb)))
               (call-next-method bkfm (flush-kanji-buffer bkfm))
               (mend bkfm fb bp)))
           (insert ()
             (let* ((fb (fred-buffer bkfm)) (bp (buffer-position fb)))
               (call-next-method)
               (mend bkfm fb bp)))
           (set-char (ch in-kanjip)
             (with-accessors ((buffer kanji-buffer)
                              (index kanji-buffer-index)
                              (size kanji-buffer-size)) bkfm
               (and in-kanjip
                    (>= index size)
                    (flush))
               (setf (char buffer (incf index)) ch
                     (slot-value bkfm 'in-kanjip) in-kanjip))))
    (cond ((null ch) (flush))
          ((stringp ch) (insert))
          ((in-kanjip bkfm)
           (set-char ch nil)
           (with-accessors ((kfpa kanji-fred-prefix-argument)
                            (buffer kanji-buffer)
                            (index kanji-buffer-index)) bkfm
             (if kfpa (let* ((ch1 (char buffer (1- index)))
                             (ch2 (char buffer index)))
                        (dotimes (x (1- kfpa))
                          (set-char ch1 t)
                          (set-char ch2 nil))
                        (setf kfpa nil)))))
          ((kanji1p ch) (set-char ch t))
          ((>= (kanji-buffer-index bkfm) 0)
           (flush)
           (insert))
          (t (insert)))))

(defmethod window-null-event-handler :after ((kfwm kanji-fred-window-mixin))
  (or (minusp (kanji-buffer-index kfwm))
      (in-kanjip kfwm)
      (progn (ed-insert-char kfwm nil)
             (fred-update kfwm))))

(defmethod window-null-event-handler :after ((kfc kanji-fred-container))
  (let ((ckh (current-key-handler kfc)))
    (if (and ckh (typep ckh 'basic-kanji-fred-mixin))
      (or (< (kanji-buffer-index ckh) 0)
          (in-kanjip ckh)
          (progn (ed-insert-char ckh nil)
                 (fred-update ckh))))))

(defmethod ed-delete-char :around ((bkfm basic-kanji-fred-mixin)
                                   &aux (fb (fred-buffer bkfm)))
  (setf (fred-prefix-argument bkfm)
        (buffer-count-kanji-char fb (fred-prefix-numeric-value bkfm) t))
  (call-next-method)
  (mend bkfm fb (buffer-position fb)))

(defmethod ed-rubout-char :around ((bkfm basic-kanji-fred-mixin)
                                   &aux (fb (fred-buffer bkfm)))
  (setf (fred-prefix-argument bkfm)
        (buffer-count-kanji-char fb (fred-prefix-numeric-value bkfm) nil))
  (call-next-method)
  (mend bkfm fb (buffer-position fb)))

(defmethod ed-forward-char :before ((bkfm basic-kanji-fred-mixin)
                                    &optional (fwd t) select
                                    &aux (fb (fred-buffer bkfm)))
  (declare (ignore select))
  (setf (fred-prefix-argument bkfm)
        (buffer-count-kanji-char fb (fred-prefix-numeric-value bkfm) fwd)))

(defmethod ed-next-line :after ((bkfm basic-kanji-fred-mixin)
                                &optional select
                                &aux (fb (fred-buffer bkfm)))
  (declare (ignore select))
  (if (buffer-in-kanjip fb)
    (set-mark fb (1+ (buffer-position fb)))))

(defmethod ed-forward-word :after ((bkfm basic-kanji-fred-mixin)
                                   &aux (fb (fred-buffer bkfm)))
  (if (buffer-in-kanjip fb)
    (set-mark fb (1+ (buffer-position fb)))))

(defmethod ed-backward-word :after ((bkfm basic-kanji-fred-mixin)
                                    &optional fwd
                                    &aux (fb (fred-buffer bkfm)))
  (declare (ignore fwd))
  (if (buffer-in-kanjip fb)
    (set-mark fb (1- (buffer-position fb)))))

(defmethod ed-backward-select-word :after ((bkfm basic-kanji-fred-mixin)
                                           &optional fwd)
  (declare (ignore fwd))
  (amend-selection-range bkfm))

(defmethod view-click-event-handler :after ((bkfm basic-kanji-fred-mixin)
                                            where)
  (declare (ignore where))
  (amend-selection-range bkfm))

(defmethod amend-selection-range ((bkfm basic-kanji-fred-mixin)
                                   &aux (fb (fred-buffer bkfm)) flag)
  (multiple-value-bind (b e) (selection-range bkfm)
    (when (buffer-in-kanjip fb b) (decf b) (setq flag t))
    (when (buffer-in-kanjip fb e) (incf e) (setq flag t))
    (when flag
      (set-selection-range bkfm b e)
      (fred-update bkfm))))

#|
(make-instance 'basic-kanji-fred-window :wrap-p t)

(make-instance 'basic-kanji-fred-window :kanji-buffer-size 4)

(make-instance 'kanji-fred-container :view-size #@(440 190)
  :view-subviews `(,(make-instance 'basic-kanji-fred-dialog-item
                      :view-position #@(20 20)
                      :view-size #@(400 150)
                      :allow-returns t)))
|#