;;;-*- Mode: Lisp; Package: KANJI -*-
;;;
;;;	kanji-fred.lisp ver3.2
;;;
;;;	Masaya UEDA
;;;	ueda@shpcsl.sharp.co.jp

(in-package :kanji)

(eval-when (:compile-toplevel :load-toplevel :execute)
  (export '(kanji-fred-mixin
            kanji-fred-window
            kanji-set-view-font-codes
            kanji-set-view-font
            *kanji-fred-default-font-spec*)))

; ---

(declaim (inline kanji-font-face-p
                 buffer-next-kanji-alphameric-change))

(defun kanji-font-face-p (ff)
  (declare (optimize (speed 3) (safety 0)))
  (or (zerop ff) (<= #x4000 (ash ff -16) #x41ff)))

(defun buffer-next-kanji-alphameric-change (buffer start end)
  (declare (optimize (speed 3) (safety 0)))
  (let* ((kanjip (buffer-before-kanjip buffer start))
         (size (if kanjip 2 1)))
    (incf start size)
    (loop while (< start end)
          do (if (eq kanjip (buffer-before-kanjip buffer start))
               (incf start size)
               (return)))
    (return-from buffer-next-kanji-alphameric-change
      (values start kanjip))))

; ---

(defvar *kanji-fred-default-font-spec* '("osaka" 9))

(setq *default-editor-class* 'kanji-fred-window)

(defclass kanji-fred-mixin (basic-kanji-fred-mixin)
  ((font2p :initform nil :reader font2p)
   (font-face-1 :accessor font-face-1)
   (mode-size-1 :accessor mode-size-1)
   (font-face-2 :accessor font-face-2)
   (mode-size-2 :accessor mode-size-2)))

(defclass kanji-fred-window (kanji-fred-mixin basic-kanji-fred-window) ())

; ---

(defmethod view-default-font ((kfm kanji-fred-mixin))
  *fred-default-font-spec*)

(defgeneric view-default-font-2 (kfm))

(defmethod view-default-font-2 ((kfm kanji-fred-mixin))
  *kanji-fred-default-font-spec*)

(defmethod initialize-instance :after ((kfm kanji-fred-mixin)
                                       &key
                                       (view-font (view-default-font kfm))
                                       (view-font-2 (view-default-font-2 kfm)))
  (multiple-value-bind (ff ms) (font-codes view-font)
    (setf (font-face-1 kfm) ff (mode-size-1 kfm) ms))
  (multiple-value-bind (ff ms) (font-codes view-font-2)
    (setf (font-face-2 kfm) ff (mode-size-2 kfm) ms)))

(defgeneric kanji-set-view-font-codes (kfm ff ms &optional which))

(defmethod kanji-set-view-font-codes ((kfm kanji-fred-mixin) ff ms
                                      &optional which
                                      &aux (fb (fred-buffer kfm)))
  (declare (optimize (speed 3) (safety 0)))
  (case which
    ((1) (setq which nil))
    ((2) (if (kanji-font-face-p ff)
           (setq which t)
           (return-from kanji-set-view-font-codes)))
    (otherwise (setq which (kanji-font-face-p ff))))
  (multiple-value-bind (b e) (selection-range kfm)
    (if (eql b e)
      (cond (which
             (setf (font-face-2 kfm) ff (mode-size-2 kfm) ms)
             (when (font2p kfm) (set-view-font-codes kfm ff ms)))
            (t (setf (font-face-1 kfm) ff (mode-size-1 kfm) ms)
               (unless (font2p kfm) (set-view-font-codes kfm ff ms))))
      (let (tmp kanjip)
        (loop while (< b e)
              do
              (multiple-value-setq (tmp kanjip)
                (buffer-next-kanji-alphameric-change fb b e))
              (when (eq which kanjip)
                (buffer-set-font-codes fb ff ms b tmp))
              (setq b tmp))))))

(defgeneric kanji-set-view-font (kfm fs &optional which))

(defmethod kanji-set-view-font ((kfm kanji-fred-mixin) fs
                                &optional which)
  (multiple-value-bind (ff ms) (font-codes fs)
    (kanji-set-view-font-codes kfm ff ms which)))

(defmethod flush-kanji-buffer :before ((kfm kanji-fred-mixin))
  (unless (font2p kfm)
    (set-view-font-codes kfm (font-face-2 kfm) (mode-size-2 kfm))
    (setf (slot-value kfm 'font2p) t)))

(macrolet ((string (kfm)
             `(let* ((fb (fred-buffer ,kfm)) (bp (buffer-position fb)))
                (call-next-method)
                (set-selection-range ,kfm bp)
                (kanji-set-view-font-codes ,kfm (font-face-1 ,kfm) (mode-size-1 ,kfm) 1)
                (kanji-set-view-font-codes ,kfm (font-face-2 ,kfm) (mode-size-2 ,kfm) 2)
                (collapse-selection ,kfm t)
                (fred-update ,kfm))))
  
  (defmethod ed-insert-char :around ((kfm kanji-fred-mixin) ch)
    (declare (optimize (speed 3) (safety 0)))
    (cond ((stringp ch) (string kfm))
          (t (when (and (characterp ch) (not (in-kanjip kfm)) (not (kanji1p ch))
                        (or (font2p kfm) (<= 0 (kanji-buffer-index kfm))))
               (call-next-method kfm nil)
               (set-view-font-codes kfm (font-face-1 kfm) (mode-size-1 kfm))
               (setf (slot-value kfm 'font2p) nil))
             (call-next-method))))
  
  (defmethod paste :around ((kfm kanji-fred-mixin))
    (if (and *paste-with-styles* (cdr (get-scrap :fred)))
      (call-next-method)
      (string kfm))))

; ---

(add-menu-items
 (find-menu "edit")
 (make-instance 'menu-item
   :menu-item-title "font..."
   :menu-item-action 'choose-font-dialog
   :command-key #\t))

(let ((ktxt "[cdr̝|] cdr͋łȂXgɑ΂Ă̂ݒ`BłȂXgcdr͏ɕʂ̃XgɂȂB")
      (atxt "[The Law of Cdr] Cdr is defined only for non-null lists. The cdr of any non-null list is always another list.")
      (sz '(9 10 12 14 16 18 20 24))
      dialog window afnt kfnt ftbl stbl stxt abtn kbtn)
  (labels ((select (tdi itm test &aux (tmp 0))
             (dolist (x (selected-cells tdi)) (cell-deselect tdi x))
             (cell-select tdi 0 (dolist (x (table-sequence tdi) 0)
                                  (if (funcall test x itm)
                                    (return tmp) (incf tmp)))))
           (contents (tdi &aux (fsc (first (selected-cells tdi))))
             (if fsc (cell-contents tdi fsc)))
           (ftbl-dia (item &aux (foo (contents item)))
             (if foo (set-view-font stxt foo)))
           (stbl-dia (item &aux (foo (contents item)))
             (if foo (set-view-font stxt foo)))
           (abtn-dia (item &aux fs)
             (declare (ignore item))
             (set-table-sequence ftbl afnt)
             (set-dialog-item-text stxt atxt)
             (setq fs (font-spec (font-face-1 window)
                                 (mode-size-1 window)))
             (select ftbl (car fs) #'string=)
             (select stbl (cadr fs) #'=)
             (set-view-font stxt fs))
           (kbtn-dia (item &aux fs)
             (declare (ignore item))
             (set-table-sequence ftbl kfnt)
             (set-dialog-item-text stxt ktxt)
             (setq fs (font-spec (font-face-2 window)
                                 (mode-size-2 window)))
             (select ftbl (car fs) #'string=)
             (select stbl (cadr fs) #'=)
             (set-view-font stxt fs))
           (qbtn-dia (item)
             (declare (ignore item))
             (return-from-modal-dialog t))
           (sbtn-dia (item &aux (f (contents ftbl)) (s (contents stbl)))
             (declare (ignore item))
             (multiple-value-bind (ff ms) (font-codes `(,f ,s))
               (if (typep window 'kanji-fred-mixin)
                 (kanji-set-view-font-codes window ff ms)
                 (set-view-font-codes window ff ms)))
             (fred-update window))
           (sqbtn-dia (item)
             (sbtn-dia item)
             (qbtn-dia item)))
    (dolist (font *font-list*)
      (let ((ff (font-codes font)))
        (if (kanji-font-face-p ff)
          (push font kfnt)
          (push font afnt))))
    (setq afnt (sort afnt #'string<=)
          kfnt (sort kfnt #'string<=)
          stxt (make-dialog-item
                'static-text-dialog-item
                #@(2 125) #@(252 30) atxt)
          ftbl (make-dialog-item
                'sequence-dialog-item
                #@(2 20) #@(111 100) "" #'ftbl-dia
                :table-hscrollp nil	:table-vscrollp t
                :cell-size #@(96 16)	:table-sequence afnt)
          stbl (make-dialog-item
                'sequence-dialog-item
                #@(117 20) #@(45 100) "" #'stbl-dia
                :table-hscrollp nil	:table-vscrollp t
                :cell-size #@(30 16)	:table-sequence sz)
          abtn (make-dialog-item
                'radio-button-dialog-item
                #@(2 1) #@(96 16) "Alphameric" #'abtn-dia
                :radio-button-pushed-p t)
          kbtn (make-dialog-item
                'radio-button-dialog-item
                #@(98 1) #@(64 16) "" #'kbtn-dia))
    
    (defun make-choose-font-dialog ()
      (setq dialog
            (make-instance 'dialog
              :view-position :centered 
              :view-size #@(256 158)
              :window-type :double-edge-box
              :window-show nil
              :view-subviews
              `(,ftbl ,stbl ,stxt ,abtn ,kbtn
                     ,(make-dialog-item
                       'button-dialog-item
                       #@(180 28) #@(58 16) "Quit" #'qbtn-dia)
                     ,(make-dialog-item
                       'button-dialog-item
                       #@(180 56) #@(58 16) "Set" #'sbtn-dia)
                     ,(make-dialog-item
                       'button-dialog-item
                       #@(172 86) #@(78 16) "Set & Quit" #'sqbtn-dia
                       :default-button t)))))
    
    (defun choose-font-dialog (&aux fs)
      (when (typep (setq window (front-window)) 'basic-kanji-fred-mixin)
        (if (null dialog) (make-choose-font-dialog))
        (cond ((typep window 'kanji-fred-mixin)
               (if (radio-button-pushed-p abtn)
                 (abtn-dia nil)
                 (kbtn-dia nil))
               (modal-dialog dialog nil))
              (t
               (radio-button-push abtn)
               (dialog-item-disable abtn)
               (dialog-item-disable kbtn) 
               (set-table-sequence ftbl kfnt)
               (set-dialog-item-text stxt ktxt)
               (setq fs (view-font window))
               (select ftbl (car fs) #'string=)
               (select stbl (cadr fs) #'=)
               (set-view-font stxt fs)               
               (modal-dialog dialog nil)
               (dialog-item-enable abtn)
               (dialog-item-enable kbtn)))))))